#!/usr/bin/env python


class InvertedIndex:
	'''
	Class representing an inverted search index.
	'''
	
	def __init__(self):
		self.index = {}
	
	def query(self, keywords):
		'''
		Returns the list of documents where all of the keywords can be found in.
		'''
		result = self.index[keywords[0]]
		for keyword in keywords[1:]:
			result = self.intersectLists(result, self.index[keyword])
		return result
	
	def readFromFile(self, filename):
		'''
		Reads an inverted index from disk with the given filename.
		The on-disk format is:
			keyword,doc1,doc2,doc3,...
		'''
		self.index = {}
		f = open(filename, "rU")
		for line in f:
			tokens = line.split(",")
			self.index[tokens[0]] = []
			for token in tokens[1:]:
				self.index[tokens[0]].append(token)
	
	def writeToFile(self, filename):
		'''
		Writes the represented in-memory inverted index to disk with the
		given filename.
		The on-disk format is:
			keyword,doc1,doc2,doc3,...
		'''
		f = open(filename, "w")
		keys = self.index.keys()
		keys.sort()
		for key in keys:
			f.write(key)
			for v in self.index[key]:
				f.write(",")
				f.write(v)
			f.write("\n")
		f.close()

	def processFile(self, filename):
		'''
		Adds all the tokens in the file in the index. The tokens are converted
		to lower case only.
		'''
		f = open(filename, "rU")
		for line in f:
			line = line.lower()
			tokens = tokenize(line);
			# add token to the index
			for token in tokens:
				if not self.index.has_key(token):
					self.index[token] = []
				#if not filename in index[token]:
				# optimization: we're processing all files one by one
				# so it is sufficient to check the last element of the list to
				# see if we have already added this file to this exact word in
				# the index
				if len(self.index[token]) == 0 or self.index[token][len(self.index[token])-1] <> filename:
					self.index[token].append(filename)
		f.close()
		
	def intersectLists(self, a, b):
		'''
		Returns a list of items that can be found in both, a and b
		'''
		result = []
		for item in a:
			if item in b:
				result.append(item)
		return result
		
	def tokenize(string):
		'''
		Splits the given string into different words. The word boundaries
		are given by the list of characters in the regular expression
		and correspond to the most common word boundaries in western languages.
		'''
		tokens_tmp = re.split('[ ,;:.!?/"(){}\[\]]', string)
		# filter out useless crap
		tokens = []
		for token in tokens_tmp:
			if len(token) > 1:
				tokens.append(token.strip())
		return tokens
		
