#!/usr/bin/env python

import sys
from BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer
import re
import urllib, urlparse

from InvertedIndex import *


index = InvertedIndex()

class QueryHandler(BaseHTTPRequestHandler):		
'''
HTTP request handler that handles query requests.
URLs of the form "/search?query=list of keywords" will be interpreted
as search queries and will return a list of documents containing *all*
the keywords.
All other URLs produce an input form for entering new queries.
'''
	def do_GET(self):
		'''
		Processes HTTP GET queries
		'''
		try:
			url = urlparse.urlparse(self.path)
			path = urllib.unquote(url.path)[1:]
			if path == "search":
				self.do_search(url)
			else:
				self.do_form(url)
			return
		except:
			self.send_error(500)
			
	def do_search(self, url):
		'''
		Parses the given url accordingly and uses the inverted index to return
		a list of results in HTML form.
		'''
		keywords = (urlparse.parse_qs(url.query)['query'][0]).split(' ')
		results = index.query(keywords)
		self.send_response(200)
		self.send_header('Content-type', 'text/html')
		self.end_headers()
	
	 	self.wfile.write('''
<html>
<head>
	<title>Search results</title>
</head>
<body>
	<h1>Search form</h1>
	<form action="search" method="GET">
	<input type="text" name="query" value="''' + urlparse.parse_qs(url.query)['query'][0] + '''"/>
	<input type="submit" value="Submit" />
	</form>
	<h1>Search results</h1>
	<ul>''')
		for result in results:
			self.wfile.write('		<li>' + result + '</li>\n')
		self.wfile.write('''
	</ul>
</body>
</html>
''')
		return

	def do_form(self, url):
		'''
		Simply sends an input form to the browser that will invoke
		the correct URL upon submit.
		'''
		self.send_response(200)
		self.send_header('Content-type', 'text/html')
		self.end_headers()
		self.wfile.write('''
<html>
<head>
	<title>Search results</title>
</head>
<body>
	<h1>Search form</h1>
	<form action="search" method="GET">
	<input type="text" name="query" />
	<input type="submit" value="Submit" />
	</form>
</body>
</html>
''')

	
			
def main(args):
	try:
		print "Reading database..."
		index.readFromFile(args[2])
		print "Done"
		print "Starting webserver"
		server = HTTPServer(('', int(args[1])), QueryHandler)
		server.serve_forever()
	except KeyboardInterrupt:
		print "User interrupt: Shutting down"
		server.socket.close()


if __name__ == "__main__":
	main(sys.argv)
